// This is a sample of the code used for testing and debuging the
// VME-SIO-4. The short and meaningless names used for the pointers to the
// Registers  come from the  Zilog Z16C30 Product Specifations Databook.
// This is one way to eliminate  Confusion between two people when
// referencing these registers.

#include <dos.h>
#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include "leelib.h"
#include "leevmeio.h"
#include "screen.h"
#include "busmgr.h"
#include "bdaddrs.h"

#define word unsigned int
#define dword unsigned long
#define ulong unsigned long
#define byte unsigned char

word stop = 0;
word CurX=0;
word CurY=0;
char kbchar;
word ReturnedVector,InterruptCalled=0;
char MyStack[256];
void ManuallyControlRxReqPin(void);
void ZilogInterruptTest(word Zilog_Offset_Addrs);
void MasterInterruptTest(word Zilog_Offset_Addrs);
void DMATestToUSC(word Zilog_Offset_Addrs);
void DMATestFromUSC(word Zilog_Offset_Addrs);
void ZilogRegisterTest(word Zilog_Offset_Addrs);
void Test_Tx_FIFOs_Of_All_Channels(void);
void InterruptHandler(ulong Vector);
FILE *FilePtr;

main()
{
  if ((FilePtr = fopen("vmesio.log","w")) == NULL)
   { cprintf("Log file not written");
   }

 ResetVme();
 ulong Zilog_Offset_Addrs = CH0_ZILOG_OFFSET;
 gotoxy(CurX,CurY++);
// cprintf("Code Has been Modified by Lee G.");
// while(kbhit()) getch();
// getch();
 do
 {
  for (int channel_cntr = 0; channel_cntr < 4; channel_cntr++) //was < 4 for all
   { switch(channel_cntr)
     { case 0:  Zilog_Offset_Addrs = CH0_ZILOG_OFFSET; break;
       case 1:  Zilog_Offset_Addrs = CH1_ZILOG_OFFSET; break;
       case 2:  Zilog_Offset_Addrs = CH2_ZILOG_OFFSET; break;
       case 3:  Zilog_Offset_Addrs = CH3_ZILOG_OFFSET; break;
       default: Zilog_Offset_Addrs = CH0_ZILOG_OFFSET; break;
     }

// The next line, normally commented out, is used to test a specific channel.
//     Zilog_Offset_Addrs = CH2_ZILOG_OFFSET;

     Test_Tx_FIFOs_Of_All_Channels();          // Test 1
     ZilogRegisterTest(Zilog_Offset_Addrs);    // Test 2
     ZilogInterruptTest(Zilog_Offset_Addrs);   // Test 3
     MasterInterruptTest(Zilog_Offset_Addrs);  // Test 4
     DMATestToUSC(Zilog_Offset_Addrs);         // Test 5
     DMATestFromUSC(Zilog_Offset_Addrs);       // Test 6

     if (kbhit())
      { stop = 1;
        fclose(FilePtr);
        while(kbhit()) getch();
        return(0);
      }
   }
 }while (!stop);
 fclose(FilePtr);
 while(kbhit()) getch();
 return(0);
}

void ZilogRegisterTest(word Zilog_Offset_Addrs)
{ // This routine is to test accesses to the various Zilog Registers
  REGISTER *ZilogBCR = new REGISTER(((Zilog_Offset_Addrs & 0x0300) + BaseAddrs),
                           A16,D16,FilePtr);
  Zilog_Offset_Addrs += BaseAddrs;
  int channel = ((Zilog_Offset_Addrs & 0x0180) >> 7);
  word ErrorFlag=0,Errors;
  static ulong total_errors=0;

  REGISTER *MstrBdCtrl = new REGISTER(MSTR_BD_CTRL,A16,D16,FilePtr);
  REGISTER *CCARL      = new REGISTER((Zilog_Offset_Addrs | CCARL_OFFSET)   ,A16,D16,FilePtr);
  REGISTER *CCARH      = new REGISTER((Zilog_Offset_Addrs | CCARH_OFFSET)   ,A16,D16,FilePtr);
  REGISTER *CMCRL      = new REGISTER((Zilog_Offset_Addrs | CMCRL_OFFSET),A16,D16,FilePtr);
  REGISTER *CMCRH      = new REGISTER((Zilog_Offset_Addrs | CMCRH_OFFSET),A16,D16,FilePtr);

  MstrBdCtrl->WriteRegister(0x01);
  ZilogBCR->WriteRegister(0x0);
  CCARL->WriteRegister(0x0);
  CCARH->WriteRegister(0x0);

  CurX=CurY=3;
  clrscr();
  gotoxy(CurX,CurY++);
  printf(" Testing Zilog Registers.......Channel %u",channel);
  gotoxy(CurX,CurY++);
  printf(" Register          Errors");
  gotoxy(CurX,CurY++);

  Errors = MstrBdCtrl->TestBits(0x00FE,0x00FE,1);
  total_errors += Errors;
  if (Errors)
   ErrorFlag = 1;
  printf(" MstrBdCtrl ---> %5d",Errors);
  gotoxy(CurX,CurY++);

  Errors = CMCRL->TestBits(0x00FF,0x00FF,1);
  total_errors += Errors;
  if (Errors)
   ErrorFlag = 1;
  printf(" CMCRL --------> %5d",Errors);
  gotoxy(CurX,CurY++);

  Errors = CMCRH->TestBits(0x00FF,0x00FF,1);
  total_errors += Errors;
  if (Errors)
   ErrorFlag = 1;
  printf(" CMCRH --------> %5d",Errors);
  printf("\nTotal Errors ----> %5d",total_errors);

  CLR_KB;
  if (ErrorFlag)
  {
   printf("\n   Errors detected...");
   delay(750);
   getch(); // pause on errors
   }
  else
   delay(750);

  delete CMCRL; delete CCARL; delete MstrBdCtrl;
  delete CMCRH; delete CCARH; delete ZilogBCR;
}

void Test_Tx_FIFOs_Of_All_Channels(void)
{ REGISTER *MstrBdCtrl    = new REGISTER(MSTR_BD_CTRL,A16,D16);
  REGISTER *Chan0Fifo     = new REGISTER(CHAN_0_FIFO,A16,D16);
  REGISTER *Chan1Fifo     = new REGISTER(CHAN_1_FIFO,A16,D16);
  REGISTER *Chan2Fifo     = new REGISTER(CHAN_2_FIFO,A16,D16);
  REGISTER *Chan3Fifo     = new REGISTER(CHAN_3_FIFO,A16,D16);
  REGISTER *Chan0FifoCtrlStat = new REGISTER(CHAN_0_FIFOSTAT,A16,D16);
  REGISTER *Chan1FifoCtrlStat = new REGISTER(CHAN_1_FIFOSTAT,A16,D16);
  REGISTER *Chan2FifoCtrlStat = new REGISTER(CHAN_2_FIFOSTAT,A16,D16);
  REGISTER *Chan3FifoCtrlStat = new REGISTER(CHAN_3_FIFOSTAT,A16,D16);

  word status;
  word NWords = 0x3FF;
  word CurX,CurY;
  word LoopCntr = 0;
  word Expected_Status = 0x00CC;
  word Error_Flag = 0;

  // These writes are to clear/reset the Tx/Rx FIFOs             //
  MstrBdCtrl->WriteRegister(0x1);
  Chan0FifoCtrlStat->WriteRegister(0x3);
  Chan1FifoCtrlStat->WriteRegister(0x3);
  Chan2FifoCtrlStat->WriteRegister(0x3);
  Chan3FifoCtrlStat->WriteRegister(0x3);

  CurX=CurY=4;
  clrscr();
  gotoxy(CurX,CurY);
  //           check status of FIFOs after reset        //

  for ( LoopCntr = 0; LoopCntr < 5; LoopCntr++)
    { clrscr();
      gotoxy(CurX,CurY);
      printf("\nTesting Fifo Status registers...Writing to Transmit FIFOs:\n");

      status = *Chan0FifoCtrlStat->mAddrPtr;
      printf("\nChan 0 Fifo Status     %04X   (Should be %04X)\n", status,
                                                         Expected_Status);
      if((*Chan0FifoCtrlStat->mAddrPtr & 0x00FF) != Expected_Status)
        {printf("Chan Fifo Failed Status Check...\n");
         Error_Flag = 1;
        }

      status = *Chan1FifoCtrlStat->mAddrPtr;
      printf("Chan 1 Fifo Status     %04X   (Should be %04X)\n", status,
                                                       Expected_Status);
      if((*Chan1FifoCtrlStat->mAddrPtr & 0x00FF) != Expected_Status)
        {printf("Chan 1 Fifo Failed Status Check...\n");
         Error_Flag = 1;
        }

      status = *Chan2FifoCtrlStat->mAddrPtr;
      printf("Chan 2 Fifo Status     %04X   (Should be %04X)\n", status,
                                                       Expected_Status);
      if((*Chan2FifoCtrlStat->mAddrPtr & 0x00FF) != Expected_Status)
        {printf("Chan 2 Fifo Failed Status Check...\n");
         Error_Flag = 1;
        }

      status = *Chan3FifoCtrlStat->mAddrPtr;
      printf("Chan 3 Fifo Status     %04X   (Should be %04X)\n", status,
                                                       Expected_Status);
      if((*Chan3FifoCtrlStat->mAddrPtr & 0x00FF) != Expected_Status)
        {printf("Chan 3 Fifo Failed Status Check...\n");
         Error_Flag = 1;
        }

      if (Error_Flag)
        {printf("\nErrors Detected in FIFO Status Checks...");
//         getch();
         Error_Flag = 0;
        }

      delay(800);

      switch(LoopCntr)
        {case 1:  Expected_Status  = 0x00CF;
                  NWords = 0x7801 ; break;
         case 2:  Expected_Status  = 0x00CB;
                  NWords = 0x3FF; break;
         case 3:  Expected_Status  = 0x00C3;
                  NWords = 0x1; break;
         default: Expected_Status  = 0x00CD; break;
        }

      for (int t = 0; t<NWords; t++)
       { wr_vme_a16s_d16(Chan0Fifo->mAddress, t);
         wr_vme_a16s_d16(Chan1Fifo->mAddress, t);
         wr_vme_a16s_d16(Chan2Fifo->mAddress, t);
         wr_vme_a16s_d16(Chan3Fifo->mAddress, t);
       }
    }
  delete Chan0Fifo;     delete Chan1Fifo;
  delete Chan2Fifo;     delete Chan3Fifo;
  delete Chan0FifoCtrlStat; delete Chan1FifoCtrlStat;
  delete Chan2FifoCtrlStat; delete Chan3FifoCtrlStat;
  delete MstrBdCtrl;
}

void ManuallyControlRxReqPin(void)
{ REGISTER *IOCRH   = new REGISTER(GENERIC_IOCRH,A16,D16);
  *IOCRH->mAddrPtr = 0x0;
  while (!kbhit())
   { printf("Setting High      %04X\n",*IOCRH->mAddrPtr );
     *IOCRH->mAddrPtr = 0x3;
     delay(1000);
     printf("Setting Low       %04X\n",*IOCRH->mAddrPtr );
     *IOCRH->mAddrPtr = 0x2;
   }
  delete IOCRH;
}


void DMATestFromUSC(word Zilog_Offset_Addrs)
{ Zilog_Offset_Addrs += BaseAddrs;
  ulong Chan_Ctrl_Addrs = BaseAddrs +
                            (((Zilog_Offset_Addrs & 0x0180) >> 6) | 0x10);
  int channel = ((Zilog_Offset_Addrs & 0x0180) >> 7);

  REGISTER *MstrBdCtrl       = new REGISTER(MSTR_BD_CTRL,A16,D16);
  REGISTER *ChanCtrl         = new REGISTER( Chan_Ctrl_Addrs      ,A16,D16);
  REGISTER *ChanFifoCtrlStat = new REGISTER((Chan_Ctrl_Addrs+0x10),A16,D16);
  REGISTER *ChanFifo         = new REGISTER((Chan_Ctrl_Addrs+0x08),A16,D16);

  REGISTER *CCARL   = new REGISTER((Zilog_Offset_Addrs | CCARL_OFFSET),A16,D16);
  REGISTER *CMCRL   = new REGISTER((Zilog_Offset_Addrs | CMCRL_OFFSET),A16,D16);
  REGISTER *CCARH   = new REGISTER((Zilog_Offset_Addrs | CCARH_OFFSET),A16,D16);
  REGISTER *CCRH    = new REGISTER((Zilog_Offset_Addrs | CCRH_OFFSET),A16,D16);
  REGISTER *IOCRH   = new REGISTER((Zilog_Offset_Addrs | IOCRH_OFFSET),A16,D16);
  REGISTER *HCRL    = new REGISTER((Zilog_Offset_Addrs | HCRL_OFFSET),A16,D16);
  REGISTER *TxRxDta = new REGISTER((Zilog_Offset_Addrs | TXRXDTA_OFFSET),A16,D16);
  REGISTER *RMRL    = new REGISTER((Zilog_Offset_Addrs | RMRL_OFFSET),A16,D16);
  REGISTER *RCSRH   = new REGISTER((Zilog_Offset_Addrs | RCSRH_OFFSET),A16,D16);
  REGISTER *RICRH   = new REGISTER((Zilog_Offset_Addrs | RICRH_OFFSET),A16,D16);
  REGISTER *TMRL    = new REGISTER((Zilog_Offset_Addrs | TMRL_OFFSET),A16,D16);
  REGISTER *TCSRH   = new REGISTER((Zilog_Offset_Addrs | TCSRH_OFFSET),A16,D16);
  REGISTER *TICRH   = new REGISTER((Zilog_Offset_Addrs | TICRH_OFFSET),A16,D16);


 MstrBdCtrl->WriteRegister(0x01);
 ChanCtrl->WriteRegister(0x53);  // Drv Lwr Cabl, Rx Lwr Cable

  // Set Bus Control Register with first write to part, be it Chan or chan 2.
  wr_vme_a16s_d16((Zilog_Offset_Addrs & 0x0300),0x0);


 CCARL->WriteRegister(0x00); // Set Usc Ctrl Reg to 0.
 CCARH->WriteRegister(0x00); // Normal Mode
 CCARH->WriteRegister(0x03); // Internal Loopback
 CMCRL->WriteRegister(0x11); // Clock controls
 RCSRH->WriteRegister(0x70); // Set Select IRCR Word Level
 RICRH->WriteRegister(0x1); // Set RxReq to start with five words rcvd
 TCSRH->WriteRegister(0x70); // Set Select IRCR Fifo Requset DMA Level
 TICRH->WriteRegister(0x01); // Set TxReq to start with five words rcvd
 HCRL->WriteRegister(0x44); // Set RxACK and TxAck as Inputs
 IOCRH->WriteRegister(0x05); // Set RxReq and TxAck as Outputs

  word ReadBack;
  static ulong NumOfPass=0;
  unsigned int NWords=0x50;

  clrscr();

  for (int temp = 0; temp < 5; temp++)
   { gotoxy(1,1);
     printf("Test Number %08ld   For Channel %d\n",++NumOfPass,channel);
     ReadBack = *ChanFifoCtrlStat->mAddrPtr;
     ReadBack &= 0x00FF;
     printf("Before Dma\n");
     printf("Chan Fifo Status -> %04X",ReadBack);
     clreol();

//     if (ReadBack != 0x00CC)
//      { printf("  <<--- Error   Expected 0x00CC.\n");
//        kbchar = getch();
//        while (kbhit()) getch();
//      }
//     else
      printf("\n");

     RMRL->WriteRegister(0x02); // Enable Receive
     TMRL->WriteRegister(0x02); // Enable Transmit

     for (word Tmp=0;Tmp<NWords;Tmp++)
       ChanFifo->WriteRegister(Tmp);    // Write data

     delay(400);

     ReadBack = *ChanFifoCtrlStat->mAddrPtr;
     ReadBack &= 0x00FF;
     printf("After Dma\n");
     printf("Chan Fifo Status -> %04X",ReadBack);
     clreol();

//     if (ReadBack != 0x00DC)
//      { printf("  <<--- Error   Expected 0x00DC.\n");
//        kbchar = getch();
//        while (kbhit()) getch();
//       }
//     else
      printf("\n");
      if (kbhit()) getchar;

     printf("Checking Rx Dta for Errors.\n");
     int ErrorCnt;
     ErrorCnt=0;
     word DTmp;
     for (Tmp=0;Tmp<NWords;Tmp++)
      { DTmp = *ChanFifo->mAddrPtr;
        if ((unsigned char)DTmp != (unsigned char)Tmp)
         { ErrorCnt++;
           if (ErrorCnt < 4)
            printf("Wrote %02X   Read %02X\n",(unsigned char)Tmp,(unsigned char)DTmp);
         }
      }

     printf("Number of Errors      %02d.\n",ErrorCnt);

     ReadBack = *ChanFifoCtrlStat->mAddrPtr;
     ReadBack &= 0x00FF;
     printf("After Read back\n");
     printf("Chan Fifo Status -> %04X\n",ReadBack);
     clreol();
     if (ReadBack != 0x00CC)
      { printf("  <<--- Error   Expected 0x00CC.\n");
//        kbchar = getch();
//        while (kbhit()) getch();
      }
     else
      printf("\n");
    if (ErrorCnt)
     { kbchar = getch();
       while (kbhit()) getch();
      }
    if(kbhit()) return;
   }
  delete TMRL; delete RICRH; delete RCSRH; delete RMRL; delete TxRxDta;
  delete HCRL; delete IOCRH; delete CCRH; delete CCARH; delete CMCRL;
  delete CCARL; delete ChanCtrl; delete MstrBdCtrl;
  delete ChanFifoCtrlStat; delete TCSRH; delete ChanFifo; delete TICRH;
}

void DMATestToUSC(word Zilog_Offset_Addrs)
{ word nWords = 5;
  word ReadValue;
  Zilog_Offset_Addrs += BaseAddrs;
  ulong Chan_Ctrl_Addrs = BaseAddrs +
                            (((Zilog_Offset_Addrs & 0x0180) >> 6) | 0x10);
  int channel = ((Zilog_Offset_Addrs & 0x0180) >> 7);

  REGISTER *ChanCtrl         = new REGISTER( Chan_Ctrl_Addrs      ,A16,D16);
  REGISTER *ChanFifoCtrlStat = new REGISTER((Chan_Ctrl_Addrs+0x10),A16,D16);
  REGISTER *ChanFifo         = new REGISTER((Chan_Ctrl_Addrs+0x08),A16,D16);

  REGISTER *BCR     = new REGISTER((Zilog_Offset_Addrs & 0xF300),A16,D16);
  REGISTER *CCARL   = new REGISTER((Zilog_Offset_Addrs | CCARL_OFFSET),A16,D16);
  REGISTER *CCARH   = new REGISTER((Zilog_Offset_Addrs | CCARH_OFFSET),A16,D16);
  REGISTER *CCRH    = new REGISTER((Zilog_Offset_Addrs | CCRH_OFFSET),A16,D16);
  REGISTER *CMCRL   = new REGISTER((Zilog_Offset_Addrs | CMCRL_OFFSET),A16,D16);
  REGISTER *IOCRH   = new REGISTER((Zilog_Offset_Addrs | IOCRH_OFFSET),A16,D16);
  REGISTER *HCRL    = new REGISTER((Zilog_Offset_Addrs | HCRL_OFFSET),A16,D16);
  REGISTER *TxRxDta = new REGISTER((Zilog_Offset_Addrs | TXRXDTA_OFFSET),A16,D16);
  REGISTER *RMRL    = new REGISTER((Zilog_Offset_Addrs | RMRL_OFFSET),A16,D16);
  REGISTER *RICRH   = new REGISTER((Zilog_Offset_Addrs | RICRH_OFFSET),A16,D16);
  REGISTER *RCSRH   = new REGISTER((Zilog_Offset_Addrs | RCSRH_OFFSET),A16,D16);
  REGISTER *TCCRL   = new REGISTER((Zilog_Offset_Addrs | TCCRL_OFFSET),A16,D16);
  REGISTER *TMRL    = new REGISTER((Zilog_Offset_Addrs | TMRL_OFFSET),A16,D16);
  REGISTER *TCSRH   = new REGISTER((Zilog_Offset_Addrs | TCSRH_OFFSET),A16,D16);
  REGISTER *TICRH   = new REGISTER((Zilog_Offset_Addrs | TICRH_OFFSET),A16,D16);
  REGISTER *MstrBdCtrl = new REGISTER(MSTR_BD_CTRL,A16,D16);

  // When the Zilog 16c30 asserts the TxReqDma, it will continue to assert it
  // until the Tx fifo is filled with 32 byte of data. The Uscctrlr should stop
  // the DMA tranfers when the Master fifo is empty. This will not deassert the
  // Zilogs TxReqDma.

  wr_vme_a16s_d16(MstrBdCtrl->mAddress, 0x01);
  wr_vme_a16s_d16(ChanCtrl->mAddress, 0x53); // Drv Lwr Cabl, Rx Lwr Cable

  // Set Bus Control Register with first write to part, be it Chan or chan 2.
  wr_vme_a16s_d16(BCR->mAddress, 0x00);
  wr_vme_a16s_d16(CCARL->mAddress, 0x00);  // Set Usc Ctrl Reg to 0.
  wr_vme_a16s_d16(CCARH->mAddress, 0x00); // Normal Mode
//  *CCRH->mAddrPtr  = 0x20; // Set TxReq to wait on Trigger Channel Load
                             // Wait for Trigger is for certain modes only
//  *CCARH->mAddrPtr = 0x03; // Internal Loopback
  wr_vme_a16s_d16(CMCRL->mAddress, 0x11); // Clock controls
  wr_vme_a16s_d16(RMRL->mAddress, 0x00); // Disable Receive
  wr_vme_a16s_d16(TMRL->mAddress, 0x00); // Disable Transmit
  wr_vme_a16s_d16(HCRL->mAddress, 0x40); // Set TxACK as Input
  word ReadBack;
  static ulong NumOfPass=0;

  clrscr();
  for (int loopcntr = 0; loopcntr < 100; loopcntr++)
   { gotoxy(1,1);
     printf("DMA Data to Zilog Test Number %08ld for Channel %d\n",++NumOfPass,channel);

     wr_vme_a16s_d16(TCSRH->mAddress, 0x70); // Set Select IRCR Fifo Requset DMA Level
     wr_vme_a16s_d16(TICRH->mAddress, 0x04); // Set TxReq to start with five words rcvd
     wr_vme_a16s_d16(RCSRH->mAddress, 0x50); // Set Select Fifo Status
     wr_vme_a16s_d16(TCSRH->mAddress, 0x50); // Set Select Fifo Status
     wr_vme_a16s_d16(IOCRH->mAddress, 0x07); // Set TxReq as Output and set RxReq High

     for (word Tmp=0;Tmp<nWords;Tmp++)
      wr_vme_a16s_d16(ChanFifo->mAddress, Tmp);    // Write data

     printf("After Writing Dta And Before Dma\n");

     rd_vme_a16s_d16(TICRH->mAddress, &ReadValue);
     ReadValue &= 0x00FF;
     printf("Chan Zilog Tx Fifo Status (Should Equal %02X ) -> %02X\n",((nWords > 32)?0:(32-nWords)),ReadValue);
     rd_vme_a16s_d16(RICRH->mAddress, &ReadValue);
     ReadValue &= 0x00FF;
     printf("Chan Zilog Rx Fifo Count  (Should Equal 0x00 ) -> %02X\n",ReadValue);
     rd_vme_a16s_d16(ChanFifoCtrlStat->mAddress, &ReadValue);
     ReadValue &= 0x00FF;
     printf("Chan Fifo Status -> %04X",ReadValue);
     clreol();

     int ShouldBe;
     if (nWords <= 32)
      ShouldBe = 0x00CC;
     else
      { ShouldBe = 0x00CD;
        if (nWords > 1024)
         { ShouldBe = 0x00CF;
           if (nWords > 0x7FFF)
            ShouldBe = 0x00CB;
         }
      }
     if (ReadValue != ShouldBe)
      { printf("  <<--- Error   Expected %04X.\n",ShouldBe);
        kbchar = getch();
        while (kbhit()) getch();
      }
     else
      printf("\n");

     wr_vme_a16s_d16(RMRL->mAddress, 0x02); // Enable Receive
     wr_vme_a16s_d16(TMRL->mAddress, 0x02); // Enable Transmit

     printf("Waiting on Tx Fifo Empty.\n");
     rd_vme_a16s_d16(ChanFifoCtrlStat->mAddress, &ReadValue);
     ReadValue &= 0x0001;
     while ((ReadValue == 0x1) && (!kbhit()))
      { rd_vme_a16s_d16(ChanFifoCtrlStat->mAddress, &ReadValue);
        ReadValue &= 0x0001;
      }

     wr_vme_a16s_d16(RMRL->mAddress, 0x0); // Disable Receive
     wr_vme_a16s_d16(TMRL->mAddress, 0x0); // Disable Transmit

     rd_vme_a16s_d16(ChanFifoCtrlStat->mAddress, &ReadValue);
     ReadValue &= 0x00FF;
     printf("After Execution of DMA to Zilog and Transmition of Data\n");
     printf("Chan Fifo Status -> %04X",ReadValue);
     clreol();
     if (ReadValue != 0x00CC)
      { printf("  <<--- Error   Expected 0x00CC.\n");
        kbchar = getch();
        while (kbhit()) getch();
      }
     else
      printf("\n");

     word MaxPrintOut=5;
     word Read_Data;
     for (Tmp=0;Tmp < nWords;Tmp++)
      { rd_vme_a16s_d16(TxRxDta->mAddress, &Read_Data);
        Read_Data &= 0x00FF;
        if ((Read_Data != Tmp) && MaxPrintOut)
         { MaxPrintOut--;
           printf("Word %3d .....  Dta in Rx Fifo-> %02X\n",Tmp,Read_Data);
         }
      }
     while (MaxPrintOut--) printf("\n");

     printf("After Read back\n");
     printf("Chan Zilog Tx Fifo Status (Should Equal 0x20 ) -> %02X\n",(*TICRH->mAddrPtr & 0x00FF));
     printf("Chan Zilog Rx Fifo Count  (Should Equal 0x00 ) -> %02X\n",(*RICRH->mAddrPtr & 0x00FF));
     printf("Chan Fifo Status -> %04X",(*ChanFifoCtrlStat->mAddrPtr & 0x00FF));
     clreol();
     if ((*ChanFifoCtrlStat->mAddrPtr & 0x00FF) != 0x00CC)
      { printf("  <<--- Error   Expected 0x00CC.\n");
        kbchar = getch();
        while (kbhit()) getch();
      }
     else
      printf("\n");
      if (kbhit()) getchar;
//    delay(1000);
   }

   *IOCRH->mAddrPtr = 0x0F; // Set TxReq and RxReq High
   delete MstrBdCtrl; delete ChanFifoCtrlStat;
   delete ChanFifo; delete ChanCtrl; delete CCARL;
   delete CCARH;    delete CCRH;     delete CMCRL;
   delete IOCRH;    delete HCRL;     delete TxRxDta;
   delete RMRL;     delete RCSRH;    delete RICRH;
   delete TMRL;     delete TICRH;    delete TCCRL;
   delete TCSRH;    delete BCR;
}

void MasterInterruptTest(word Zilog_Offset_Addrs)
{ Zilog_Offset_Addrs += BaseAddrs;
  ulong Chan_Ctrl_Addrs = BaseAddrs +
                            (((Zilog_Offset_Addrs & 0x0180) >> 6) | 0x10);
  int channel = ((Zilog_Offset_Addrs & 0x0180) >> 7);

  REGISTER *MstrBdCtrl        = new REGISTER(MSTR_BD_CTRL,A16,D16);
  REGISTER *MasterICR         = new REGISTER(MASTER_ICR,A16,D16);
  REGISTER *MasterISR         = new REGISTER(MASTER_ISR,A16,D16);
  REGISTER *MasterIVR         = new REGISTER(MASTER_IVR,A16,D16);
  REGISTER *ChanCtrl          = new REGISTER( Chan_Ctrl_Addrs      ,A16,D16);
  REGISTER *ChanFifo          = new REGISTER((Chan_Ctrl_Addrs+0x08),A16,D16);
  REGISTER *ChanFifoCtrlStat  = new REGISTER((Chan_Ctrl_Addrs+0x10),A16,D16);
  REGISTER *MasterSync        = new REGISTER((Chan_Ctrl_Addrs+0x18),A16,D16);

  REGISTER *CCARL             = new REGISTER((Zilog_Offset_Addrs | CCARL_OFFSET) ,A16,D16);
  REGISTER *CMCRL             = new REGISTER((Zilog_Offset_Addrs | CMCRL_OFFSET) ,A16,D16);
  REGISTER *CCARH             = new REGISTER((Zilog_Offset_Addrs | CCARH_OFFSET) ,A16,D16);
  REGISTER *CCSRH             = new REGISTER((Zilog_Offset_Addrs | CCSRH_OFFSET) ,A16,D16);
  REGISTER *CCRH              = new REGISTER((Zilog_Offset_Addrs | CCRH_OFFSET) ,A16,D16);
  REGISTER *IOCRH             = new REGISTER((Zilog_Offset_Addrs | IOCRH_OFFSET) ,A16,D16);
  REGISTER *HCRL              = new REGISTER((Zilog_Offset_Addrs | HCRL_OFFSET) ,A16,D16);
  REGISTER *TxRxDta           = new REGISTER((Zilog_Offset_Addrs | TXRXDTA_OFFSET) ,A16,D16);
  REGISTER *RMRL              = new REGISTER((Zilog_Offset_Addrs | RMRL_OFFSET) ,A16,D16);
  REGISTER *RCSRL             = new REGISTER((Zilog_Offset_Addrs | RCSRL_OFFSET) ,A16,D16);
  REGISTER *RCSRH             = new REGISTER((Zilog_Offset_Addrs | RCSRH_OFFSET) ,A16,D16);
  REGISTER *RICRH             = new REGISTER((Zilog_Offset_Addrs | RICRH_OFFSET) ,A16,D16);
  REGISTER *TMRL              = new REGISTER((Zilog_Offset_Addrs | TMRL_OFFSET) ,A16,D16);
  REGISTER *TCSRL             = new REGISTER((Zilog_Offset_Addrs | TCSRL_OFFSET) ,A16,D16);
  REGISTER *TCSRH             = new REGISTER((Zilog_Offset_Addrs | TCSRH_OFFSET) ,A16,D16);
  REGISTER *TICRH             = new REGISTER((Zilog_Offset_Addrs | TICRH_OFFSET) ,A16,D16);

  MstrBdCtrl->WriteRegister(0x01);
  MstrBdCtrl->WriteRegister(0x02);

  ChanCtrl->WriteRegister(0x53);  // Drv Lwr Cabl, Rx Lwr Cable
  MasterIVR->WriteRegister(0xAF);  // Write master vector register
  MasterSync->WriteRegister(0x03);

  // Set Bus Control Register with first write to part, be it Chan or chan 2.
  wr_vme_a16s_d16((Zilog_Offset_Addrs & 0x0300),0x0);
  CCARL->WriteRegister(0x00); // Set Usc Ctrl Reg to 0.
  CCARH->WriteRegister(0x00); // Normal Mode
//  *CCARH->mAddrPtr = 0x03;  // Internal Loopback
  wr_vme_a16s_d16(CMCRL->mAddress,0x11);  // Clock controls
  wr_vme_a16s_d16(HCRL->mAddress,0x04);   // Set RxACK as Input
  wr_vme_a16s_d16(IOCRH->mAddress,0x01);  // Set RxReq as Output
  InterruptCalled=0x00;
  word ReadBack;
  static ulong NumOfPass=0;

  EpcSetIntr(5,InterruptHandler, &MyStack[0], 0);

  clrscr();
  setcursor(INVISIBLE);
  for (int loopcntr = 0;loopcntr < 100; loopcntr++)
   { // En Intr on Rx Sync Word
      switch((Zilog_Offset_Addrs - BaseAddrs))
       { case CH0_ZILOG_OFFSET: MasterICR->WriteRegister(0x0001); break;
         case CH1_ZILOG_OFFSET: MasterICR->WriteRegister(0x0004); break;
         case CH2_ZILOG_OFFSET: MasterICR->WriteRegister(0x0010); break;
         case CH3_ZILOG_OFFSET: MasterICR->WriteRegister(0x0040); break;
         default:               MasterICR->WriteRegister(0x0001); break;
       }

     wr_vme_a16s_d16(RCSRH->mAddress,0x70);  // Set Select IRCR Fifo Request DMA Level
     wr_vme_a16s_d16(RICRH->mAddress,0x04);  // Set RxReq to start with five words rcvd
     wr_vme_a16s_d16(RCSRH->mAddress,0x50);  // Set Select Fifo Status
     wr_vme_a16s_d16(TCSRH->mAddress,0x50);  // Set Select Fifo Status
     gotoxy(1,1);
     printf("Test Number %08ld              Channel...%d\n",++NumOfPass,channel);
     ReadBack = *ChanFifoCtrlStat->mAddrPtr;
     ReadBack &= 0x00FF;
     printf("Before Dma\n");
     printf("Chan Fifo Status -> %04X",ReadBack);
     clreol();

     if (ReadBack != 0x00CC)
      { printf("  <<--- Error   Expected 0x00CC.\n");
        kbchar = getch();
        while (kbhit()) getch();
      }
     else
      printf("\n");

     for (word Tmp=0;Tmp<5;Tmp++)
      TxRxDta->WriteRegister(Tmp);    // Write data

     printf("After Writing Dta\n");
     printf("Chan Zilog Tx Fifo Status     (Should Equal 0x1B) -> %02X\n",(*TICRH->mAddrPtr & 0x00FF));
     printf("Chan Zilog Rx Fifo Count      (Should Equal Zero) -> %02X\n",(*RICRH->mAddrPtr & 0x00FF));

     RMRL->WriteRegister(0x02); // Enable Receive
     TMRL->WriteRegister(0x02); // Enable Transmit

     delay(5);

     RMRL->WriteRegister(0x00); // Disable Receive
     TMRL->WriteRegister(0x00); // Disable Transmit

     printf("After Dma\n");
     printf("Chan Zilog Tx Fifo Status     (Should Equal 0x20) -> %02X\n",(*TICRH->mAddrPtr & 0x00FF));
     printf("Chan Zilog Rx Fifo Count      (Should Equal Zero) -> %02X\n",(*RICRH->mAddrPtr & 0x00FF));

     ReadBack = *ChanFifoCtrlStat->mAddrPtr;
     ReadBack &= 0x00FF;
     printf("Chan Fifo Status -> %04X",ReadBack);
     clreol();

     if (ReadBack != 0x00DC)
      { printf("  <<--- Error   Expected 0x00DC.\n");
        kbchar = getch();
        while (kbhit()) getch();
       }
     else
      printf("\n");

     CLR_KB;
     if (!InterruptCalled)
      printf("Waiting on Interrupt:      Status Register %04X",
                                                   *MasterISR->mAddrPtr);
     while ((!InterruptCalled) && !kbhit());

     gotoxy(1,wherey());
     clreol();
     printf("Got Interrupt:      Status Register %04X\n",
                                                   *MasterISR->mAddrPtr);
     for (Tmp=0;Tmp<5;Tmp++)
      printf("Dta in Rx Fifo-> %04X\n",*ChanFifo->mAddrPtr);

     MasterICR->WriteRegister(0x000); // Clear Intr Control Register
//      MasterISR->WriteRegister(0x100); // Clear Intr Status Register
     wr_vme_a16s_d16(MasterISR->mAddress, 0xFFFF);

     ReadBack = *ChanFifoCtrlStat->mAddrPtr;
     ReadBack &= 0x00FF;
     printf("Chan Zilog Tx Fifo Status     (Should Equal 0x20) -> %02X\n",(*TICRH->mAddrPtr & 0x00FF));
     printf("Chan Zilog Rx Fifo Count      (Should Equal Zero) -> %02X\n",(*RICRH->mAddrPtr & 0x00FF));
     printf("Chan Fifo Status -> %04X\n",ReadBack);
     clreol();
     if (ReadBack != 0x00CC)
      { printf("  <<--- Error   Expected 0x00CC.\n");
        kbchar = getch();
        while (kbhit()) getch();
      }
     else
      printf("\n");

     ReturnedVector &= 0x00FF;

     printf("ReturnedVector  -> %04X     Value In Vector Register -> %04X\n",ReturnedVector,*MasterIVR->mAddrPtr);
     printf("InterruptCalled -> %02d\n",InterruptCalled);
     printf("Status Register: %04X\n",*MasterISR->mAddrPtr);

     if ((ReturnedVector != (0xA8 | channel))
       || (InterruptCalled != 0x5))
      printf("Error with Interrupt    Vector should have been  %02X\n",(0xA8 | channel));
     if (kbhit()) getchar;

     ReturnedVector = InterruptCalled = 0;
   }

  setcursor(NORMAL);
  EpcDisIntr(5);

  delete MstrBdCtrl;   delete MasterICR;    delete MasterISR;
  delete MasterIVR;    delete MasterSync;   delete ChanFifoCtrlStat;
  delete ChanFifo;     delete ChanCtrl;     delete CCARL;
  delete CMCRL;        delete CCARH;        delete CCSRH;
  delete CCRH;         delete IOCRH;        delete HCRL;
  delete TxRxDta;      delete RMRL;         delete RCSRL;
  delete RCSRH;        delete RICRH;        delete TMRL;
  delete TCSRL;        delete TCSRH;        delete TICRH;
}

void ZilogInterruptTest(word Zilog_Offset_Addrs)
{ Zilog_Offset_Addrs += BaseAddrs;
  ulong Chan_Ctrl_Addrs = BaseAddrs +
                            (((Zilog_Offset_Addrs & 0x0180) >> 6) | 0x10);
  int channel = ((Zilog_Offset_Addrs & 0x0180) >> 7);

  REGISTER *MstrBdCtrl = new REGISTER(MSTR_BD_CTRL,A16,D16);
  REGISTER *ChanCtrl   = new REGISTER(Chan_Ctrl_Addrs,A16,D16);

  REGISTER *CCARL      = new REGISTER( (Zilog_Offset_Addrs | CCARL_OFFSET)   ,A16,D16);
  REGISTER *CCARH      = new REGISTER( (Zilog_Offset_Addrs | CCARH_OFFSET)   ,A16,D16);
  REGISTER *CMCRL      = new REGISTER( (Zilog_Offset_Addrs | CMCRL_OFFSET)   ,A16,D16);
  REGISTER *CMCRH      = new REGISTER( (Zilog_Offset_Addrs | CMCRH_OFFSET)   ,A16,D16);
  REGISTER *IVRL       = new REGISTER( (Zilog_Offset_Addrs | IVRL_OFFSET)    ,A16,D16);
  REGISTER *ICRL       = new REGISTER( (Zilog_Offset_Addrs | ICRL_OFFSET)    ,A16,D16);
  REGISTER *ICRH       = new REGISTER( (Zilog_Offset_Addrs | ICRH_OFFSET)    ,A16,D16);
  REGISTER *DCCRH      = new REGISTER( (Zilog_Offset_Addrs | DCCRH_OFFSET)   ,A16,D16);
  REGISTER *RMRL       = new REGISTER( (Zilog_Offset_Addrs | RMRL_OFFSET)    ,A16,D16);
  REGISTER *TxRxDta    = new REGISTER( (Zilog_Offset_Addrs | TXRXDTA_OFFSET) ,A16,D16);
  REGISTER *TMRL       = new REGISTER( (Zilog_Offset_Addrs | TMRL_OFFSET)    ,A16,D16);
  REGISTER *TCSRL      = new REGISTER( (Zilog_Offset_Addrs | TCSRL_OFFSET)   ,A16,D16);
  REGISTER *TCSRH      = new REGISTER( (Zilog_Offset_Addrs | TCSRH_OFFSET)   ,A16,D16);
  REGISTER *TICRL      = new REGISTER( (Zilog_Offset_Addrs | TICRL_OFFSET)   ,A16,D16);
  REGISTER *TICRH      = new REGISTER( (Zilog_Offset_Addrs | TICRH_OFFSET)   ,A16,D16);

  EpcSetIntr(5,InterruptHandler, &MyStack[0], 0);

  word originalx=wherex(),originaly=wherey();
  word curX,curY;
  static ulong NumOfPass=0;
  MstrBdCtrl->WriteRegister(0x01);
  MstrBdCtrl->WriteRegister(0x03);
//  ChanCtrl->WriteRegister(0x54);   // Enable Chan USC Interrupts,
                                     // Drv Lwr Cabl, Rx Lwr Cable
  wr_vme_a16s_d16(ChanCtrl->mAddress, 0x54);
  // Set Bus Control Register with first write to part, be it Chan or chan 2.
  wr_vme_a16s_d16((Zilog_Offset_Addrs & 0x0300),0x0);

  CCARL->WriteRegister(0x00); // Set Usc Ctrl Reg to 0.
   //  *CCARH->mAddrPtr = 0x03; // Internal Loopback
  wr_vme_a16s_d16(CCARH->mAddress, 0x0);
  CMCRL->WriteRegister(0x11); // Clock controls
  wr_vme_a16s_d16(ICRH->mAddress, 0x80);
  IVRL->WriteRegister(0x45);  // Set Vector
  clrscr();
  for (int loopcntr = 0; loopcntr < 100; loopcntr++)
   { gotoxy(1,1);
     printf("Test Number %08ld              Channel...%d\n",++NumOfPass,channel);
     TICRL->WriteRegister(0x40); // Enable Tx Idle Interrupt
     ICRL->WriteRegister(0xC8);  // Set Tx Status Interrupt Enable

     word Tmp=0;
     TxRxDta->WriteRegister(Tmp++);   // Write data
     TxRxDta->WriteRegister(Tmp++);   // Write data
     TxRxDta->WriteRegister(Tmp++);   // Write data
     TxRxDta->WriteRegister(Tmp++);   // Write data
     TxRxDta->WriteRegister(Tmp++);   // Write data

     *RMRL->mAddrPtr = 0x02; // Enable Receive
      delay(20);
     *TMRL->mAddrPtr = 0x02; // Enable Transmit

     printf("Waiting on Interrupt\n");
     word Counting = 0x2000;
     while ((!InterruptCalled) && !kbhit() && Counting--);
     clreol();
     if (!InterruptCalled)
      printf("Interrupt did not occur\n");
     else
      printf("Interrupt did occur\n");

     word ReadBack[5];
     for (Tmp=0;Tmp<5;Tmp++)
      ReadBack[Tmp] = (*TxRxDta->mAddrPtr & 0x00FF);

     for (Tmp=0;Tmp<5;Tmp++)
      { printf("Read -> %02X",ReadBack[Tmp]);
//        clreol();
        if (ReadBack[Tmp] != (word)(Tmp & 0x00FF))
         { printf("  <<--- Error   Expected %02X",Tmp);
//           getch();
//           while(kbhit()) getch();
         }
        printf("\n");
      }

     *DCCRH->mAddrPtr = 0x48;   // Clear the Interrupt pending bit
     *TCSRL->mAddrPtr = 0x40;   // Clear the Interrupt Status bit
     *RMRL->mAddrPtr = 0x00;    // Disable Receive
     *TMRL->mAddrPtr = 0x00;    // Disable Transmit

     word Temp1,Temp2;
     TCSRL->ReadRegister(&Temp1);
     TCSRH->ReadRegister(&Temp2);
     printf("Lwr Tx Cmd Stat Reg -> %02X\n",(Temp1 & 0x00FF));
     printf("Upr Tx Cmd Stat Reg -> %02X\n",(Temp1 & 0x00FF));
     printf("ReturnedVector  -> %02X\n",(ReturnedVector & 0x00FF));
     printf("InterruptCalled -> %02d\n",InterruptCalled);
     if (((ReturnedVector & 0x00FF) != 0x45)
       || (InterruptCalled != 0x5))
      { printf("Error with Interrupt\n");
      }
     ReturnedVector = InterruptCalled = 0;
     delay(20);
    if(kbhit()) return;

   }
   gotoxy(originalx,originaly);
   *MstrBdCtrl->mAddrPtr = 0x01;
   delete CCARH; delete CCARL; delete CMCRH; delete CMCRL;
   delete RMRL;  delete TICRH; delete TICRL; delete TMRL;
   delete TxRxDta; delete IVRL; delete ICRL; delete ICRH;
   delete MstrBdCtrl; delete ChanCtrl; delete DCCRH; delete TCSRL;
   delete TCSRH;

   EpcDisIntr(5);
}

void InterruptHandler(ulong Vector)
{ ReturnedVector = (word)Vector;
  InterruptCalled=0x05;
}
